﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace CMDSpacePirates
{
	public class Enemy : Ship
    {
        float shootTime = 7.5f * 10000000;
        float shootTimer;

		public Enemy(Vector position) : base(position)
        {
			renderPriority = 1;
			pos = position;
			size = new Vector(4, 4);
			graphic = (
                @"┌~~┐" +
                @"/xx/" +
                @"/xx/" +
                @"└~~┘"
                ).ToCharArray();

            //pos = new Vector(78,3);
		}

		public override void Start()
        {
            base.Start();
            ApplyRandomEquipment();
            AddEquipment('g');
            Animation.PlayMe(this, anim, true);
        }

        void ApplyRandomEquipment()
        {
            int equipmentPieces = Program.random.Next(1, equipmentSpaces.Count);

            for (int i = 0; i < equipmentPieces; i++)
            {
                int r = Program.random.Next(10);
                if (r > 4)
                    AddEquipment('e');
                else
                    AddEquipment('g');
            }
        }

        public override void Tick()
        {
            base.Tick();
            shootTimer += Time.DeltaTime;
            if (shootTimer >= shootTime / guns.Count && guns.Count > 0)
            {
                FloatVector v = new FloatVector(pos.x - Program.player.ShipCenter.x, pos.y - Program.player.ShipCenter.y);
                float z = (float)Math.Pow(((float)Math.Pow(v.x, 2) + (float)Math.Pow(v.y, 2)), .5f);
                FloatVector u = new FloatVector(-(v.x/z), -(v.y/z));
                
                Shoot(u);
                shootTimer = 0;
            }
        }

        public void Shoot(FloatVector shootDir)
        {
            //Program.Destroy(this);
            FireGun();
            new Projectile(pos, shootDir);
        }

        public override void Destroy()
        {
            GameManager.points += 10;
            Render.renderForegroundColor = ConsoleColor.Yellow;
            Render.renderBackgroundColor = ConsoleColor.DarkYellow;
            //Render.renderBackgroundColor = ConsoleColor.Yellow;
            base.Destroy();
            //new AnimObject(pos, size, deathAnim);
            new AnimObject(new Vector(pos.x - 5, pos.y + 3), new Vector(12, 8), bigDeathAnim);
            //graphic = new char[] { ' ' };
            //GameConsole.Write("Enemy dead");
        }

        public char[][] anim = new char[][]
        {
                (
                @"┌-~┐" +
                @"/xx/" +
                @"\xx\" +
                @"└~-┘"
                ).ToCharArray(),
                (
                @"┌~-┐" +
                @"\xx\" +
                @"/xx/" +
                @"└-~┘"
                ).ToCharArray(),
        };

        public char[][] deathAnim = new char[][]
        {
                (
                @"┌ ~ " +
                @" x /" +
                @"\ x " +
                @" ~ ┘"
                ).ToCharArray(),
                (
                @"  ~ " +
                @"\   " +
                @"  x " +
                @"   ┘"
                ).ToCharArray()
        };

        public char[][] bigDeathAnim = new char[][]
        {
                (
                @"            " +
                @"            " +
                @"    ┌-~┐    " +
                @"    /××/    " +
                @"    \××\    " +
                @"    └~-┘    " +
                @"            " +
                @"            " 
                ).ToCharArray(),
                (
                @"            " +
                @"  ┌~     ┐  " +
                @"      ×     " +
                @"  /         " +
                @"      ×  \  " +
                @"    ×       " +
                @"  └  ~  ┘   " +
                @"            "
                ).ToCharArray(),
                (
                @" ~        ┐ " +
                @"    ×       " +
                @"/          \" +
                @"        ×   " +
                @"            " +
                @"    ×     \ " +
                @"            " +
                @"  ~     ┘   "
                ).ToCharArray(),
        };
    }
}


//graphic = (
//				 " /──<" +
//				 "<ge| " +
//				@" \──<"
//				).ToCharArray();