﻿using System;
using System.Collections.Generic;

namespace CMDSpacePirates {
	public class Ship : GameObject
    {
        public Vector ShipCenter
        {
            get
            {
                return new Vector(pos.x + size.y / 2, pos.y - size.y / 2);
            }
        }
		public Vector dir;
		public int cooldown = 9 + 48; //Always keep 48 - char reasons
		public int maxHp;
		public static char[] equipmentCharacters =
		{ '0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'g', 'e' };

		public List<int> equipmentSpaces = new List<int>();
		public List<int> guns = new List<int>();
		public List<int> engines = new List<int>();
        
		public Ship(Vector position) : base(position) {}

		public void Fire(Vector pos)
        {
			bool hit = false;
			hit = FireGun();
			if (hit)
				Program.IsCollision<Enemy>(pos);
		}

		public override void Hit(int spaceHit) {
            if (GetType() == typeof(Player))
            {
                GameConsole.Write("You were hit!");

                Render.renderForegroundColor = ConsoleColor.Red;
                Render.renderBackgroundColor = ConsoleColor.DarkRed;
                //Render.renderBackgroundColor = ConsoleColor.Red;
            }
            else
            {
                GameConsole.Write("Enemy was hit!");
                Render.renderForegroundColor = ConsoleColor.Yellow;
            }

			char equipmentPart = graphic[spaceHit];
			switch (equipmentPart) {
				case 'g':
					RemovePart(guns, spaceHit);
					break;
				case 'e':
					RemovePart(engines, spaceHit);
					break;
				default:
					RemovePart();
					break;
			}
		}

		public void RemovePart(List<int> type, int i)
        {
            //// INVINCIBILITY
            //if (GetType() == typeof(Player))
            //    return;

            type.Remove(i);
			equipmentSpaces.Add(i);
			graphic[i] = 'x';
			if (equipmentSpaces.Count >= maxHp)
            {
                if (GetType() == typeof(Player))
                    GameConsole.Write("You died");

                Program.Destroy(this);
            }
		}

		public void RemovePart(int i) {
			RemovePart(GetEquipmentType(i), i);
		}

		public void RemovePart() {
			int i = GetRandomEquipment();
			RemovePart(i);
		}

		public bool IsFullyUpgraded {
			get {
				return equipmentSpaces.Count == 0;
			}
		}

		public void AddEquipment(char equipment) {
			if (IsFullyUpgraded)
				return;

			graphic[equipmentSpaces[0]] = equipment;
			GetEquipmentType(equipment).Add(equipmentSpaces[0]);
			equipmentSpaces.RemoveAt(0);
		}

		public bool AreGunsAvailable {
			get {
				foreach (int g in guns) {
					if (graphic[g] == 'g') {
						return true;
					}
				}
				return false;
			}
		}

		public List<int> GetEquipmentType(int i)
        {
            if (graphic[i] == 'e')
                return engines;
            if (graphic[i] == 'g')
                return guns;
            if (guns.Contains(i))
				return guns;
			return null;
		}

		public List<int> GetEquipmentType(char c) {
			if (c == 'e')
				return engines;
			if (c == 'g')
				return guns;
			return null;
		}

		public int GetRandomEquipment() {
			int i = 0; //random, should be within 0 and graphic length-1
			for (int j = i; j < graphic.Length + i; j++) {
				int k = (j < graphic.Length) ? j : j - graphic.Length;
				if (IsEquipment(graphic[k])) {
					return k;
				}
			}
			throw new ApplicationException("No equipments on ship - should be dead");
		}

		public void InstantiateEquipment() {
			for (int i = 0; i < graphic.Length; i++) {
				switch (graphic[i]) {
					case 'x':
						equipmentSpaces.Add(i);
						maxHp++;
						break;
					case 'g':
						guns.Add(i);
						maxHp++;
						break;
					case 'e':
						engines.Add(i);
						maxHp++;
						break;
					default:
						break;
				}
			}
		}

		void CheckGuns() {
			//if (!AreGunsAvailable) {
				foreach (int g in guns) {
					char c = graphic[g];
					if (c != 'g' && c != 'e') {
						int v = c;

						if (c == '0') // 0
						{
							graphic[g] = 'g';
							return;
						}

						v--;
						graphic[g] = (char)v;
					}
				}
			//}
		}

		public static bool IsEquipment(char character){
			foreach (char c in equipmentCharacters)
				if (c == character)
					return true;
			return false;
		}

		public override void Start() {
			base.Start();
			this.InstantiateEquipment();
		}

		public override void Tick() {
			base.Tick();
			CheckGuns();
			pos = new Vector(pos.x + dir.x * engines.Count, pos.y + dir.y * engines.Count);
		}

		public bool FireGun()
        {
            if (guns.Count == 0)
                return false;

            int g = Program.random.Next(guns.Count);
            graphic[guns[g]] = (char)(cooldown);
            return true;

            //foreach (int g in guns)
            //         {
            //	if (graphic[g] == 'g')
            //             {
            //		graphic[g] = (char)(cooldown);
            //		return true;
            //	}
            //}
            //return false;
        }
    }
}
