﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;

namespace CMDSpacePirates
{
    static class Program
    {
        //All GameObjects in the scene
        public static List<GameObject> gameObjects = new List<GameObject>();
        public static List<GameObject> destroyObjs = new List<GameObject>();
        public static List<GameObject> addObjQueue = new List<GameObject>();
        public static Player player;
        public static Starmap starmap;
        public static bool running = true;
        public static Random random;

        public static void Main(string[] args)
        {
            //Initialize here
            StartGame();

            while (running)
            {
                while (Time.ElapsedTime > Time.NextFrame)
                {
                    Update();
                    Time.UpdateTime();
                }
                //System.Threading.Thread.Sleep(0);
            }
            //Console.Read();
        }
        
        private static void StartGame()
        {
            GameConsole.Write("type HELP for controls");

            SoundCreator.SetupSound();
            SoundCreator.MakeSound("test");

            Time.Reset();
            Render.Start();
            InstantiateScene();
        }

        private static void Update()
        {
            foreach (GameObject gameObject in gameObjects)
            {
                if (gameObject.firstFrame)
                {
                    gameObject.Start();
                    gameObject.firstFrame = false;
                }
                gameObject.Tick();
            }

            foreach (GameObject go in addObjQueue)
            {
                gameObjects.Add(go);
            }
            addObjQueue.Clear();

            foreach (GameObject go in destroyObjs)
            {
                gameObjects.Remove(go);
            }
            destroyObjs.Clear();

            //Console.WriteLine(Time.DeltaTime);

            Animation.Tick();
            GameManager.Tick();
            Render.Draw();

            //GameConsole.UserInput(Console.ReadLine() + "");
        }

        private static void InstantiateScene()
        {
            object errorSupressor; // no function in game
            player = new Player(new Vector(50, 50));
            random = new Random();
            starmap = new Starmap();
            //TESTS
            //errorSupressor = new Enemy(new Vector(70, 20)) { dir = new Vector(-1, 0) };
            errorSupressor = new Enemy(new Vector(50, 1)) { dir = new Vector(0, 0) };
        }

        public static GameObject FindNearestObject<T>(Vector position)
        {
            GameObject closestObject = null;
            float shortestDistance = 1000;
            foreach (GameObject g in gameObjects)
            {
                if (g.GetType() == typeof(T))
                {
                    float distance = Vector.Distance(g.pos, position);
                    if (distance < shortestDistance)
                    {
                        closestObject = g;
                        shortestDistance = distance;
                    }
                }
            }
            return closestObject;
        }

        public static GameObject IsCollision(Vector position, bool damage = true)
        {
            return IsCollision<GameObject>(position, damage);
        }
        public static GameObject IsCollision<T>(Vector position, bool damage = true)
        {
            // OBS - could be wrong enemy it hits
            //GameObject checker = FindNearestObject<T>(position);

            foreach (GameObject checker in gameObjects)
            {
                if (checker != null && checker is T)
                {
                    if (typeof(Enemy) == typeof(T))
                    {

                        Vector localHitCoord = Vector.DifferenceWrap(new Vector(position.x, checker.pos.y), new Vector(checker.pos.x, position.y));
                        if (localHitCoord.x >= 0 && localHitCoord.y >= 0)
                        {
                            if (localHitCoord.x < checker.size.x && localHitCoord.y < checker.size.y)
                            {
                                int collisionPosition = localHitCoord.y * checker.size.x + localHitCoord.x;
                                if (checker.collider[collisionPosition])
                                {
                                    if (damage)
                                        checker.Hit(collisionPosition);
                                    return checker;
                                }
                            }
                        }
                    }
                    else if (typeof(Player) == typeof(T))
                    {
                        Vector localHitCoord = Vector.Subtract(new Vector(position.x, checker.pos.y), new Vector(checker.pos.x, position.y));
                        if (localHitCoord.x >= 0 && localHitCoord.y >= 0)
                        {
                            if (localHitCoord.x < checker.size.x && localHitCoord.y < checker.size.y)
                            {
                                int collisionPosition = localHitCoord.y * checker.size.x + localHitCoord.x;
                                if (checker.collider[collisionPosition])
                                {
                                    if (damage)
                                        checker.Hit(collisionPosition);
                                    return checker;
                                }
                            }
                        }
                    }
                }
            }
            return null;
        }

        //public static GameObject IsCollision<T>(Vector position, bool damage = true)
        //{
        //    // OBS - could be wrong enemy it hits
        //    GameObject checker = FindNearestObject<T>(position);
        //    if (checker != null)
        //    {
        //        Vector localHitCoord = new Vector(checker.pos.x - position.x, position.y - checker.pos.y);
        //        if (localHitCoord.x >= 0 && localHitCoord.y >= 0)
        //        {
        //            if (localHitCoord.x < checker.size.x && localHitCoord.y < checker.size.y)
        //            {
        //                int collisionPosition = localHitCoord.y * checker.size.x + localHitCoord.x;
        //                if (checker.collider[collisionPosition])
        //                {
        //                    if (damage)
        //                        checker.Hit(collisionPosition);
        //                    return checker;
        //                }
        //            }
        //        }
        //    }
        //    return null;
        //}

        public static void Destroy(GameObject gameObject)
        {
            gameObject.Destroy();
            destroyObjs.Add(gameObject); // Will be destroyed when tick-loop finishes
        }

        public static void Create(GameObject gameObject)
        {
            addObjQueue.Add(gameObject); // Will be added when tick-loop finishes
        }

        public static T Clamp<T>(this T val, T min, T max) where T : IComparable<T>
        {
            if (val.CompareTo(min) < 0) return min;
            else if (val.CompareTo(max) > 0) return max;
            else return val;
        }
    }
}