﻿using System;
namespace CMDSpacePirates
{
    public struct Vector
    {
        public int x, y;

        public Vector(int _x, int _y) : this()
        {
            x = _x;
            y = _y;
        }

        public static Vector Zero
        {
            get { return new Vector(0, 0); }
        }
        public static Vector Right
        {
            get { return new Vector(1, 0); }
        }
        public static Vector Up
        {
            get { return new Vector(0, 1); }
        }

        public static Vector GetIntVector(FloatVector vector)
        {
            return new Vector((int)vector.x, (int)vector.y);
        }

        public static float Magnitude(Vector vector)
        {
            return (float)Math.Sqrt(vector.x * vector.x + vector.y * vector.y);
        }

        public static float Distance(Vector v1, Vector v2)
        {
            float distX = Math.Abs(v1.x - v2.x);
            float distY = Math.Abs(v1.y - v2.y);

            return (float)Math.Sqrt(Math.Pow(distX, 2) + Math.Pow(distY, 2));
        }

        public static Vector Subtract(Vector v1, Vector v2)
        {
            return new Vector(v1.x - v2.x, v1.y - v2.y);
        }

        public static Vector Add(Vector v1, Vector v2)
        {
            return new Vector(v1.x + v2.x, v1.y + v2.y);
        }

        public static Vector Multiply(Vector v1, Vector v2)
        {
            return new Vector(v1.x * v2.x, v1.y * v2.y);
        }

        public static Vector Multiply(Vector v, int a, bool ignoreZero = false)
        {
            if (a == 0 && ignoreZero)
                return v;
            return new Vector(v.x * a, v.y * a);
        }

        public static Vector DifferenceWrap(Vector v1, Vector v2)
        {
            Vector i = Vector.Zero;
            i.x = Math.Abs(v1.x - v2.x);
            i.y = Math.Abs(v1.y - v2.y);

            if (i.x > 500)
                i.x = Math.Max(v1.x, v2.x) - (Math.Min(v1.x, v2.x) + 1000);
            else
                i.x = Math.Max(v1.x, v2.x) - Math.Min(v1.x, v2.x);

            if (i.y > 500)
                i.y = (Math.Min(v1.y, v2.y) + 1000) - Math.Max(v1.y, v2.y);
            else
                i.y = Math.Min(v1.y, v2.y) - Math.Max(v1.y, v2.y);

            return i;
        }

        public static Vector DifferenceWrap(Vector v1, Vector v2, bool v1minusv2)
        {
            Vector i = Vector.Zero;
            i.x = Math.Abs(v1.x - v2.x);
            i.y = Math.Abs(v1.y - v2.y);

            if (i.x > 500)
                i.x = Math.Max(v1.x, v2.x) - (Math.Min(v1.x, v2.x) + 1000);
            else
                i.x = v1.x - v2.x;

            if (i.y > 500)
                i.y = (Math.Min(v1.y, v2.y) + 1000) - Math.Max(v1.y, v2.y);
            else
                i.y = v1.y - v2.y;

            return i;
        }
    }

    public struct FloatVector
    {
        public float x, y;

        public FloatVector(float _x, float _y) : this()
        {
            x = _x;
            y = _y;
        }

        public static Vector Zero
        {
            get { return new Vector(0, 0); }
        }

        public static FloatVector GetFloatVector(Vector vector)
        {
            return new FloatVector(vector.x, vector.y);
        }

        public static float Magnitude(Vector vector)
        {
            return (float)Math.Sqrt(vector.x * vector.x + vector.y * vector.y);
        }

        public static float Distance(Vector v1, Vector v2)
        {
            float distX = Math.Abs(v1.x - v2.x);
            float distY = Math.Abs(v1.y - v2.y);

            return (float)Math.Sqrt(Math.Pow(distX, 2) + Math.Pow(distY, 2));
        }

        public static FloatVector Subtract(FloatVector v1, FloatVector v2)
        {
            return new FloatVector(v1.x - v2.x, v1.y - v2.y);
        }

        public static FloatVector Add(FloatVector v1, FloatVector v2)
        {
            return new FloatVector(v1.x + v2.x, v1.y + v2.y);
        }
    }
}